#include "ra111.h"
#include "userlist.h"
#include "scrcodes.h"


void checkpath( char *s )
{
    if ( s[ strlen( s ) - 1 ] != '\\' )
	{
        strcat( s, "\\" );
	}
}


void welcome( void )
{
	cprintf( "\r\n" );
	textcolor( LIGHTCYAN );
	cprintf( "%s %s  %s  %s  Written in 1993 by D. Heering.\r\n", TITLE, VERSION,
							  __DATE__, __TIME__ );
	textcolor( YELLOW );
	cprintf( "Make LastCallers Ans/Asc/Avt screens. FreeWare .\r\n\r\n" );
	textcolor( LIGHTGRAY );
}


void usage( char *name )
{
	cprintf( "    Usage: %s <report name> <number of entries> [Hidden] [Local]\r\n\r\n", TITLE );
	cprintf( "    Sample: %s NEWS 15 L H\r\n\r\n", name );
	cprintf( "           Makes NEWS.ASC, NEWS.ANS, NEWS.AVT in RA's text path.\r\n" );
	cprintf( "           Includes last 15 users, local and hidden users included.\r\n\r\n" );
	exit( 0 );
}


void gettimestamp( int fh, char time_string[ 21 ] )
{
   struct ftime ft;

   getftime( fh, &ft );
   sprintf( time_string, "%02u:%02u:%02u %02u/%02u/%04u",
		  ft.ft_hour, ft.ft_min,   ft.ft_tsec * 2,
		  ft.ft_day,  ft.ft_month, ft.ft_year+1980 );
}


int writefile( int fh, char *buf, int length )
{
	int result;

	result = write( fh, buf, length );
	*buf = '\0';
	return result;
}


void calctime( char *s, word *online )
{
	char line[ 80 ];
	char time[ 6 ];
	char timestring[ 20 ];

	if ( *online < 60 )
		sprintf( timestring, "%u minute%c", *online, ( *online < 2 ) ? ' ' : 's' );
	else
	{
		time[0] = *online / 600;
		time[1] = ( *online - time[0] * 600 ) / 60;
		time[2] = ':';
		time[3] = ( *online - time[0] * 600 - time[1] * 60 ) / 10;
		time[4] = ( *online - time[0] * 600 - time[1] * 60 - time[3] * 10 );
		time[0] += '0';
		time[1] += '0';
		time[3] += '0';
		time[4] += '0';
		time[5] = '\0';
		sprintf( timestring, "%s hour%c", time, ( *online < 119 ) ? ' ' : 's' );
	}
	sprintf( line, REP_USER_FOOT, timestring );
	strcat( s, line );
}


void writespeeds( int *fh, int *speeds, int records )
{
	char line[ 255 ];
	int teller;
	const unsigned int bps[] = {   300u,  1200u,  2400u,  4800u,  7200u,
								  9600u, 12000u, 14400u, 16800u, 38400u,
								 57600u };

	for ( teller = 0; teller < 12; teller++ )
		if ( speeds[ teller ] != 0 )
		{
			sprintf( line, " %3d caller%s %5u BPS - %04.1f%%\r\n", speeds[ teller ], ( speeds[ teller ] == 1 ) ? " at " : "s at", bps[ teller ],  ((float)speeds[ teller ] / (float)records)*100.00 );
			write( *fh, line, strlen( line ) );
		}
	if ( speeds[ 12 ] != 0 )
	{
		sprintf( line, " %3d local                - %04.1f%%\r\n", speeds[ teller ], ((float)speeds[ teller ] / (float)records)*100.0 );
		write( *fh, line, strlen( line ) );
	}
}


void write_header( const char *name, int *fh, char time[ 21 ], int recs, int type )
{
	char line[255];	/* Output string buffer */
	line[ 0 ] = '\0';

	/* ClearScreen, set color, Report title */
	if ( type == ANS )
	{
		ANSI_cls( line );
		ANSI_fg_color( line, S_WHITE );
	}
	if ( type == ASC )
		ASCII_cls( line );
	if ( type == AVT )
	{
		AVT_cls( line );
		AVT_fg_color( line, S_WHITE );
	}
	writefile( *fh, line, strlen( line ) );
	sprintf( line, REP_TITLE, recs, name, time );

	/* Userlist header */
	if ( type == ANS )
		ANSI_fg_color( line, S_LIGHTCYAN );
	if ( type == AVT )
		AVT_fg_color( line, S_LIGHTCYAN );
	writefile( *fh, line, strlen( line ) );
	sprintf( line, REP_USER_HEAD, "Name", "City", "BPS", "LogOn", "LogOff", "Times" );

	/* Userlist line */
	if ( type == ANS )
	{
		ANSI_fg_color( line, S_LIGHTGREEN );
		ASCII_line( line, 73 );
		ANSI_fg_color( line, S_CYAN );
	}
	if ( type == AVT )
	{
		AVT_fg_color( line, S_LIGHTGREEN );
		AVT_line( line, 73 );
		AVT_fg_color( line, S_CYAN );
	}
	if ( type == ASC )
		ASCII_line( line, 73 );
	writefile( *fh, line, strlen( line ) );
}


void write_footer( int *fh, int *speeds, int records, word *online, int type )
{
	char line[255];	/* Output string buffer */
	line[ 0 ] = '\0';


	/* Userlist line */
	if ( type == ANS )
	{
		ANSI_fg_color( line, S_LIGHTGREEN );
		ASCII_line( line, 73 );
		ANSI_fg_color( line, S_MAGENTA );
	}
	if ( type == AVT )
	{
		AVT_fg_color( line, S_LIGHTGREEN );
		AVT_line( line, 73 );
		AVT_fg_color( line, S_MAGENTA );
	}
	if ( type == ASC )
		ASCII_line( line, 73 );
	writefile( *fh, line, strlen( line ) );

	/* Tijd statistiek */
	calctime( line, online );
	writefile( *fh, line, strlen( line ) );

	/* BPS overzicht header */
	if ( type == ANS )
		ANSI_fg_color( line, S_LIGHTMAGENTA );
	if ( type == AVT )
		AVT_fg_color( line, S_LIGHTMAGENTA );
	strcat( line, REP_BPS_HEAD );

	/* BPS line */
	if ( type == ANS )
	{
		ANSI_fg_color( line, S_LIGHTBLUE );
		ASCII_line( line, 33 );
		ANSI_fg_color( line, S_GREEN );
	}
	if ( type == AVT )
	{
		AVT_fg_color( line, S_LIGHTBLUE );
		AVT_line( line, 33 );
		AVT_fg_color( line, S_GREEN );
	}
	if ( type == ASC )
		ASCII_line( line, 33 );
	writefile( *fh, line, strlen( line ) );

	/* Speed statistieken */
	writespeeds( fh, speeds, records );

	/* BPS line */
	if ( type == ANS )
	{
		ANSI_fg_color( line, S_LIGHTBLUE );
		ASCII_line( line, 33 );
		ANSI_fg_color( line, S_LIGHTGRAY );
	}
	if ( type == AVT )
	{
		AVT_fg_color( line, S_LIGHTBLUE );
		AVT_line( line, 33 );
		AVT_fg_color( line, S_LIGHTGRAY );
	}
	if ( type == ASC )
		ASCII_line( line, 33 );
	writefile( *fh, line, strlen( line ) );

	sprintf( line, REP_BPS_FOOT, records, ( records == 1 ) ? " " : "s " );
	writefile( *fh, line, strlen( line ) );
	sprintf( line, REP_COPYRITE, TITLE, VERSION );
	writefile( *fh, line, strlen( line ) );
	sprintf( line, REP_PRESS );
	writefile( *fh, line, strlen( line ) );
}
