/**
 ** Program: UserList
 ** Author : Donald Heering
 ** Version: 1.0  - Initial release
 **          1.01 - Bugfix: opened CONFIG.RA in text mode, must be binary..
 **                 Added test for CONFIG.RA revision number.
 **          1.02 - Forgot to check the System path for trailing '\'
 **                 Now opens CONFIG.RA in DENYWRITE mode (networks)
 **
 ** Disclaimer: The author of UserList will not be liable for any damages
 **             resulting from the use, or inability to use, the UserList
 **             program and any of it's source code.
 **
 **             The source code and program are free. The author does
 **             not pretend this source is usefull for educational
 **             purposes. (He did not use any piece of paper when conceiving
 **             this program, which really is a bad habit! Oh well.. ;)
 **/

#include "ra111.h"
#include "userlist.h"


int main( int argc, char **argv )
{
	/* Ringbuffer declaraties */
	LASTCALL    *buf_ring;
	int         *buf_first;
	int         *buf_last;

    char *ra;               /* RA environment variable */
    word rarev;             /* CONFIG.RA revision */
	char config[81]; 		/* Config.Ra */
    char rapath[81];        /* Gegevens van callers */
    char anspath[81];       /* Ansi report */
    char avtpath[81];       /* Avatar report */
    char ascpath[81];       /* Ascii report */
    char system[31];        /* BBS name */
    char progname[81];      /* Program name */
    int  incl_hidden = 0;   /* Hidden users includen? */
    int  incl_local  = 0;   /* Local users includen? */
    int  entries = 0;

	int fh_ra  = -1;	/* File handles input en output files */
	int fh_ans = -1;
	int fh_avt = -1;
	int fh_asc = -1;

	enum boolean { FALSE, TRUE };

	int 		teller;		  /* Algemeen tellertje voor lussen */
	void 		*buffer;	  /* Buffer waar inhoud LASTCALL.BBS in komt */
	LASTCALL    *users;       /* Pointer om langs de buffer te lopen */
	int         speeds[13];   /* Speed statistieken */
	word        *online;	  /* Totale inlogtijd */
	int 		include;	  /* Dit record includen? */
	long 		filesize; 	  /* Lengte van de data file */
	int 		bytes;		  /* Hoeveelheid bytes ingelezen van LASTCALL.BBS */
	int         records;      /* Het totaal aantal records in LASTCALL.BBS */
	int			records_read; /* Het aantal records in de buffer */
	char		dated[ 21 ];  /* Tijdstip waarop LASTCALL.BBS verwerkt is */

	directvideo = 0;

	/* Minimaal DOS 3.x */
	if ( _osmajor < 3 )
	{
		cprintf( "Sorry, this program needs MS-DOS version 3.x or higher..\n" );
		exit( 1 );
	}

	/* Actuele naam van het programma, inclusief path */
	argv[ 0 ][ (int ) strlen( argv[ 0 ] ) - 4 ] = '\0';
    strcpy (progname, argv[ 0 ]);

	/* Say 'Hello' */
	welcome();

	/* Lees RA env. var. */
	ra = getenv( "RA" );
	if ( ra == NULL )
	{
		cprintf( "RA environment variable not found..\r\n" );
		exit( 1 );
	}

	/* Lees paden uit CONFIG.RA */
	strcpy( config, ra );
    checkpath( config );
	strcat( config, "CONFIG.RA" );
    fh_ra = fileopen( config, O_RDONLY| O_BINARY| O_DENYWRITE, 0 );
    read( fh_ra, &rarev, sizeof( rarev ) );
	if ( rarev < 0x0110 )
    {
        cprintf( "%s %s requires RA 1.1x or higher!\n", TITLE, VERSION );
        exit( 1 );
    }
	if ( lseek( fh_ra, 1056L, SEEK_SET ) == -1 )
		perror( config );
    read( fh_ra, rapath, 80 );
    P2Cconvert( rapath );
    checkpath( rapath );
	if ( lseek( fh_ra, 812L, SEEK_SET ) == -1 )
		perror( config );
    read( fh_ra, ascpath, 80 );
    P2Cconvert( ascpath );
    checkpath( ascpath );
	if ( lseek( fh_ra, 1258L, SEEK_SET ) == -1 )
		perror( config );
    read( fh_ra, system, 30 );
    P2Cconvert( system );
	if ( fh_ra != -1 )
	{
		close( fh_ra );
		fh_ra = -1;
	}

    strcat( rapath, "LASTCALL.BBS" );
    strcat( ascpath, argv[ 1 ] );
    strcpy( anspath, ascpath );
    strcpy( avtpath, ascpath );
    strcat( ascpath, ".ASC" );
    strcat( anspath, ".ANS" );
    strcat( avtpath, ".AVT" );

	/* Verwerk command line argumenten */
	if ( argc < 3 )
        usage( progname );
	else
	{
        entries = atoi( argv[ 2 ] );
		if ( ( toupper( *argv[ 3 ] ) == 'H' ) || ( toupper( *argv[ 4 ] ) == 'H' ) )
            incl_hidden = 1;
		if ( ( toupper( *argv[ 3 ] ) == 'L' ) || ( toupper( *argv[ 4 ] ) == 'L' ) )
            incl_local = 1;
	}

	/* Alloceer memory voor online */
	online = (word *)malloc( sizeof( word ) );
	*online = 0;

	buf_first = (int *)malloc( sizeof( int ) );
	*buf_first = 0;

	buf_last = ( int * )malloc( sizeof( int ) );
	*buf_last = 0;

	/* Initialiseer speeds array */
	for ( teller = 0; teller < 13; teller++)
		speeds[ teller ] = 0;

	/* Alloceer buffer om LASTCALL.BBS in te lezen */
	buffer = (void*)malloc( BUF_SIZE );
	if ( buffer == NULL )
	{
		cprintf( "Cannot allocate memory for buffer!\r\n" );
		exit( 1 );
	}

	/* Alloceer ring buffer om de laatste xx users te onthouden */
    buf_ring = (LASTCALL *)malloc( (entries + 1) * SIZE );
	if ( buf_ring == NULL )
	{
		cprintf( "Cannot allocate memory for ring buffer!\r\n" );
		exit( 1 );
	}

    fh_ra = fileopen( rapath, O_BINARY| O_RDONLY| O_DENYNONE, 0 );

	filesize = filelength( fh_ra );
	records = (int) filesize / SIZE;

	cprintf( "Processing LASTCALL.BBS..\r\n" );

	/* Schrijf users naar de ringbuffer */
	do
	{
		bytes = read( fh_ra, buffer, BUF_SIZE );
		users = buffer;
		records_read = bytes / SIZE;

        for ( teller = 0; teller < records_read; teller++ )
		{
			include = FALSE;
            if ( incl_local && incl_hidden )
				include = TRUE;
			if ( ( users[ teller ].baud != 0 ) && ( ( users[ teller ].attribute & 1 ) == 0 ) )
				include = TRUE;
            if ( ( users[ teller ].baud == 0 ) && incl_local && ( ( users[ teller ].attribute & 1 ) == 0 ) )
				include = TRUE;
            if ( ( ( users[ teller ].attribute & 1 ) != 0 ) && incl_hidden && ( users[ teller ].baud != 0 ) )
				include = TRUE;
			if ( include )
			{
				update_speeds( users[ teller ].baud, speeds );
				update_online( &users[ teller ], online );
                update_ring( &users[ teller ], buf_ring, buf_first, buf_last, entries );
			}
			else
			{
                records--;
			}
		}
	} while ( records_read != 0 );

	if ( records != 0 )
	{
		cprintf( "Writing screens to disk..\r\n" );

		/* Probeer output files te creren */
        fh_ans = fileopen( anspath, O_CREAT| O_TRUNC| O_RDWR| O_BINARY, S_IFREG| S_IREAD| S_IWRITE );
        fh_avt = fileopen( avtpath, O_CREAT| O_TRUNC| O_RDWR| O_BINARY, S_IFREG| S_IREAD| S_IWRITE );
        fh_asc = fileopen( ascpath, O_CREAT| O_TRUNC| O_RDWR| O_BINARY, S_IFREG| S_IREAD| S_IWRITE );

		gettimestamp( fh_ra, dated );

		/* Schrijf headers */
        write_header( system, &fh_ans, dated,
            ( records > entries) ? entries : records, ANS );
        write_header( system, &fh_avt, dated,
            ( records > entries) ? entries : records, AVT );
        write_header( system, &fh_asc, dated,
            ( records > entries) ? entries : records, ASC );

        write_ring( &fh_ans, buf_ring, buf_first, buf_last, entries );
        write_ring( &fh_avt, buf_ring, buf_first, buf_last, entries );
        write_ring( &fh_asc, buf_ring, buf_first, buf_last, entries );

		write_footer( &fh_ans, speeds, records, online, ANS );
		write_footer( &fh_avt, speeds, records, online, AVT );
		write_footer( &fh_asc, speeds, records, online, ASC );
	}

	if ( fh_ans != -1 )
		close( fh_ans );
	if ( fh_asc != -1 )
		close( fh_asc );
	if ( fh_avt != -1 )
		close( fh_avt );

	if ( records == 0 )
	{
		cprintf( "No records written, deleting old screens..\r\n" );
        unlink( anspath );
        unlink( ascpath );
        unlink( avtpath );
	}

	if ( fh_ra != -1 )
		close( fh_ra );

	cprintf( "Done..\r\n\r\n" );

	return 0;
} /* main() */


int fileopen( char *path, int access, unsigned mode )
{
	int fh;
	fh = open( path, access, mode );
	if ( fh == -1)
	{
		perror( path );
		exit( 1 );
	}
	return fh;
}


void update_speeds( word baud, int *speeds )
{
	switch ( baud )
	{
		case   300U:
			speeds[0]++;
			break;
		case  1200U:
			speeds[1]++;
			break;
		case  2400U:
			speeds[2]++;
			break;
		case  4800U:
			speeds[3]++;
			break;
		case  7200U:
			speeds[4]++;
			break;
		case  9600U:
			speeds[5]++;
			break;
		case 12000U:
			speeds[6]++;
			break;
		case 14400U:
			speeds[7]++;
			break;
		case 16800U:
			speeds[8]++;
			break;
		case 19200U:
			speeds[9]++;
			break;
		case 38400U:
			speeds[10]++;
			break;
		case 57600U:
			speeds[11]++;
			break;
		case     0U:
			speeds[12]++;
			break;
	}
}


void update_online( LASTCALL *user, word *online )
{
	int on;
	int off;
	int used;

	on   = 600 * ( user->logon[1] - '0' );
	on  += 60  * ( user->logon[2] - '0' );
	on  += 10  * ( user->logon[4] - '0' );
	on  +=       ( user->logon[5] - '0' );
	off  = 600 * ( user->logoff[1] - '0' );
	off += 60  * ( user->logoff[2] - '0' );
	off += 10  * ( user->logoff[4] - '0' );
	off +=       ( user->logoff[5] - '0' );
	used = off - on;
	if ( used < 0 )
		used += 1440;
	*online += (word)used;
}


void update_ring( LASTCALL *user, LASTCALL *buf_ring, int *buf_first, int *buf_last, int entries )
{
	int  position;      /* Where to write in ring buffer */

	if ( *buf_last == 0 && *buf_first == 0 )
	{
		(*buf_last)++;
		position = 0;
	}
	else if ( *buf_last > *buf_first )
        if ( *buf_last == entries )
		{
            position = entries;
			*buf_last = 0;
			*buf_first = 1;
		}
		else
		{
			position = *buf_last;
			(*buf_last)++;
		}
	else
        if ( *buf_first == entries )
		{
			position = *buf_last;
			(*buf_last)++;
			*buf_first = 0;
		}
		else
		{
			position = *buf_last;
			(*buf_last)++;
			(*buf_first)++;
		}

	buf_ring[ position ] = *user;
}


void write_ring( int *fh, LASTCALL *buf_ring, int *buf_first, int *buf_last, int entries )
{
	LASTCALL user;
	char line[ 128 ];
	char baud[ 6 ];	/* Baud string */
	int position;   /* Position to write to file */
	int first = *buf_first;
	int last  = *buf_last;

	do
	{
		if ( last > first )
		{
			position = first;
			first++;
		}
		else
		{
			position = first;
            if ( first == entries )
				first = 0;
			else
				first++;
		}

		user = buf_ring[ position ];
		P2Cconvert( user.name );
		P2Cconvert( user.city );
		P2Cconvert( user.logon );
		P2Cconvert( user.logoff );

		if ( user.baud == 0 )
			sprintf( baud, "%-5s", "Local" );
		else
			sprintf( baud, "%-5u", user.baud );

		sprintf( line, REP_USER, user.name,
			user.city, baud, user.logon,
			user.logoff, user.times );

		write( *fh, line, strlen( line ) );
	} while ( first != last );
}
